@@ -43,6 +43,8 @@
 #include <rak/path.h>
 #include <torrent/exceptions.h>
 #include <torrent/torrent.h>
+#include <torrent/tracker.h>
+#include <torrent/tracker_list.h>
 
 #include "utils/variable_generic.h"
 
@@ -50,39 +52,54 @@
 
 namespace core {
 
-Download::Download(torrent::Download d) :
+Download::Download(download_type d) :
   m_download(d),
+  m_fileList(d.file_list()),
+  m_trackerList(d.tracker_list()),
+
+  m_hashFailed(false),
 
   m_chunksFailed(0) {
 
   m_connTrackerSucceded = m_download.signal_tracker_succeded(sigc::bind(sigc::mem_fun(*this, &Download::receive_tracker_msg), ""));
-  m_connTrackerFailed = m_download.signal_tracker_failed(sigc::mem_fun(*this, &Download::receive_tracker_msg));
-  m_connStorageError = m_download.signal_storage_error(sigc::mem_fun(*this, &Download::receive_storage_error));
+  m_connTrackerFailed   = m_download.signal_tracker_failed(sigc::mem_fun(*this, &Download::receive_tracker_msg));
+  m_connStorageError    = m_download.signal_storage_error(sigc::mem_fun(*this, &Download::receive_storage_error));
 
   m_download.signal_chunk_failed(sigc::mem_fun(*this, &Download::receive_chunk_failed));
 
-  m_variables.insert("connection_current", new utils::VariableSlotString<const char*, const std::string&>(rak::mem_fn(this, &Download::connection_current),
-													  rak::mem_fn(this, &Download::set_connection_current)));
-  m_variables.insert("connection_leech",   new utils::VariableAny(connection_type_to_string(torrent::Download::CONNECTION_LEECH)));
-  m_variables.insert("connection_seed",    new utils::VariableAny(connection_type_to_string(torrent::Download::CONNECTION_SEED)));
-  m_variables.insert("state",              new utils::VariableBencode(&m_download.bencode(), "rtorrent", "state", torrent::Bencode::TYPE_STRING));
-  m_variables.insert("tied_to_file",       new utils::VariableBencode(&m_download.bencode(), "rtorrent", "tied_to_file", torrent::Bencode::TYPE_STRING));
+  m_variables.insert("connection_current", new utils::VariableStringSlot(rak::mem_fn(this, &Download::connection_current),
+									 rak::mem_fn(this, &Download::set_connection_current)));
 
-  m_variables.insert("directory",          new utils::VariableSlotString<>(rak::mem_fn(&m_download, &torrent::Download::root_dir),
-									   rak::mem_fn(this, &Download::set_root_directory)));
+  m_variables.insert("connection_leech",   new utils::VariableAny(connection_type_to_string(download_type::CONNECTION_LEECH)));
+  m_variables.insert("connection_seed",    new utils::VariableAny(connection_type_to_string(download_type::CONNECTION_SEED)));
 
-  m_variables.insert("min_peers",          new utils::VariableSlotValue<uint32_t, uint32_t>(rak::mem_fn(&m_download, &torrent::Download::peers_min),
-											    rak::mem_fn(&m_download, &torrent::Download::set_peers_min),
-											    "%u"));
-  m_variables.insert("max_peers",          new utils::VariableSlotValue<uint32_t, uint32_t>(rak::mem_fn(&m_download, &torrent::Download::peers_max),
-											    rak::mem_fn(&m_download, &torrent::Download::set_peers_max),
-											    "%u"));
-  m_variables.insert("max_uploads",        new utils::VariableSlotValue<uint32_t, uint32_t>(rak::mem_fn(&m_download, &torrent::Download::uploads_max),
-											    rak::mem_fn(&m_download, &torrent::Download::set_uploads_max),
-											    "%u"));
-  m_variables.insert("priority",           new utils::VariableSlotValue<uint32_t, uint32_t>(rak::mem_fn(this, &Download::priority),
-											    rak::mem_fn(this, &Download::set_priority),
-											    "%u"));
+  // 0 - stopped
+  // 1 - started
+  m_variables.insert("state",              new utils::VariableObject(bencode(), "rtorrent", "state", torrent::Object::TYPE_VALUE));
+  m_variables.insert("complete",           new utils::VariableObject(bencode(), "rtorrent", "complete", torrent::Object::TYPE_VALUE));
+
+  // 0 - Not hashing
+  // 1 - Normal hashing
+  // 2 - Download finished, hashing
+  m_variables.insert("hashing",            new utils::VariableObject(bencode(), "rtorrent", "hashing", torrent::Object::TYPE_VALUE));
+  m_variables.insert("tied_to_file",       new utils::VariableObject(bencode(), "rtorrent", "tied_to_file", torrent::Object::TYPE_STRING));
+
+  // The "state_changed" variable is required to be a valid unix time
+  // value, it indicates the last time the torrent changed its state,
+  // resume/pause.
+  m_variables.insert("state_changed",      new utils::VariableObject(bencode(), "rtorrent", "state_changed", torrent::Object::TYPE_VALUE));
+
+  m_variables.insert("directory",          new utils::VariableStringSlot(rak::mem_fn(&m_fileList, &torrent::FileList::root_dir),
+									 rak::mem_fn(this, &Download::set_root_directory)));
+
+  m_variables.insert("min_peers",          new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::peers_min),
+									rak::mem_fn(&m_download, &download_type::set_peers_min)));
+  m_variables.insert("max_peers",          new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::peers_max),
+									rak::mem_fn(&m_download, &download_type::set_peers_max)));
+  m_variables.insert("max_uploads",        new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::uploads_max),
+									rak::mem_fn(&m_download, &download_type::set_uploads_max)));
+  m_variables.insert("priority",           new utils::VariableValueSlot(rak::mem_fn(this, &Download::priority),
+									rak::mem_fn(this, &Download::set_priority)));
 }
 
 Download::~Download() {
@@ -93,35 +110,24 @@ Download::~Download() {
   m_connTrackerFailed.disconnect();
   m_connStorageError.disconnect();
 
-  m_download = torrent::Download();
-}
-
-void
-Download::start() {
-  if (is_done())
-    m_download.set_connection_type(string_to_connection_type(m_variables.get("connection_seed").as_string()));
-  else
-    m_download.set_connection_type(string_to_connection_type(m_variables.get("connection_leech").as_string()));
-
-  // Update the priority to ensure it has the correct
-  // seeding/unfinished modifiers.
-  set_priority(priority());
-  m_download.start();
+  m_download = download_type();
 }
 
 void
 Download::enable_udp_trackers(bool state) {
-  for (int i = 0, last = m_download.size_trackers(); i < last; ++i)
-    if (m_download.tracker(i).tracker_type() == torrent::Tracker::TRACKER_UDP)
+  torrent::TrackerList tl = m_download.tracker_list();
+
+  for (int i = 0, last = tl.size(); i < last; ++i)
+    if (tl.get(i).tracker_type() == torrent::Tracker::TRACKER_UDP)
       if (state)
-	m_download.tracker(i).enable();
+	tl.get(i).enable();
       else
-	m_download.tracker(i).disable();
+	tl.get(i).disable();
 }
 
 uint32_t
 Download::priority() {
-  return get_bencode().get_key("rtorrent").get_key("priority").as_value();
+  return bencode()->get_key("rtorrent").get_key("priority").as_value();
 }
 
 void
@@ -135,14 +141,7 @@ Download::set_priority(uint32_t p) {
   else
     torrent::download_set_priority(m_download, p * p);
 
-  get_bencode().get_key("rtorrent").insert_key("priority", (int64_t)p);
-}
-
-void
-Download::receive_finished() {
-  m_download.set_connection_type(string_to_connection_type(m_variables.get("connection_seed").as_string()));
-  // FIXME
-  //torrent::download_set_priority(m_download, 2);
+  bencode()->get_key("rtorrent").insert_key("priority", (int64_t)p);
 }
 
 void
@@ -158,23 +157,23 @@ Download::receive_storage_error(std::string msg) {
   m_message = "Storage error: [" + msg + "]";
 }
 
-torrent::Download::ConnectionType
+Download::download_type::ConnectionType
 Download::string_to_connection_type(const std::string& name) {
   // Return default if the name isn't found.
   if (name == "leech")
-    return torrent::Download::CONNECTION_LEECH;
+    return download_type::CONNECTION_LEECH;
   else if (name == "seed")
-    return torrent::Download::CONNECTION_SEED;
+    return download_type::CONNECTION_SEED;
   else
     throw torrent::input_error("Unknown peer connection type selected: \"" + name + "\"");
 }
 
 const char*
-Download::connection_type_to_string(torrent::Download::ConnectionType t) {
+Download::connection_type_to_string(download_type::ConnectionType t) {
   switch (t) {
-  case torrent::Download::CONNECTION_LEECH:
+  case download_type::CONNECTION_LEECH:
     return "leech";
-  case torrent::Download::CONNECTION_SEED:
+  case download_type::CONNECTION_SEED:
     return "seed";
   default:
     return "unknown";
@@ -211,6 +210,28 @@ Download::priority_to_string(uint32_t p) {
   }
 }
 
+float
+Download::distributed_copies() const {
+  const uint8_t* avail = m_download.chunks_seen();
+  const uint8_t* end = avail + m_download.chunks_total();
+
+  if (avail == NULL)
+    return 0;
+
+  int minAvail = std::numeric_limits<uint8_t>::max();
+  int num = 0;
+
+  for (; avail < end; ++avail)
+    if (*avail == minAvail) {
+      num++;
+    } else if (*avail < minAvail) {
+      minAvail = *avail;
+      num = 1;
+    }
+
+  return minAvail + 1 - (float)num / m_download.chunks_total();
+}
+
 void
 Download::receive_chunk_failed(__UNUSED uint32_t idx) {
   m_chunksFailed++;
@@ -220,17 +241,17 @@ Download::receive_chunk_failed(__UNUSED uint32_t idx) {
 void
 Download::set_root_directory(const std::string& path) {
   if (path.empty()) {
-    m_download.set_root_dir("./" + (m_download.size_file_entries() > 1 ? m_download.name() : std::string()));
+    m_fileList.set_root_dir("./" + (m_fileList.size() > 1 ? m_download.name() : std::string()));
 
   } else {
     std::string fullPath = rak::path_expand(path);
 
-    m_download.set_root_dir(fullPath +
+    m_fileList.set_root_dir(fullPath +
 			    (*fullPath.rbegin() != '/' ? "/" : "") +
-			    (m_download.size_file_entries() > 1 ? m_download.name() : ""));
+			    (m_fileList.size() > 1 ? m_download.name() : ""));
   }
 
-  m_download.bencode().get_key("rtorrent").insert_key("directory", path);
+  bencode()->get_key("rtorrent").insert_key("directory", path);
 }
 
 }
