@@ -36,12 +36,14 @@
 
 #include "config.h"
 
+#include <cstdlib>
 #include <fstream>
 #include <sstream>
 #include <stdexcept>
 #include <rak/path.h>
-#include <torrent/bencode.h>
+#include <torrent/object.h>
 #include <torrent/exceptions.h>
+#include <torrent/rate.h>
 
 #include "utils/variable_generic.h"
 
@@ -52,6 +54,7 @@
 
 #include "download.h"
 #include "download_factory.h"
+#include "download_store.h"
 
 namespace core {
 
@@ -69,9 +72,9 @@ DownloadFactory::DownloadFactory(const std::string& uri, Manager* m) :
   m_taskLoad.set_slot(rak::mem_fn(this, &DownloadFactory::receive_load));
   m_taskCommit.set_slot(rak::mem_fn(this, &DownloadFactory::receive_commit));
 
-  m_variables.insert("connection_leech", new utils::VariableAny(control->variables()->get("connection_leech")));
-  m_variables.insert("connection_seed",  new utils::VariableAny(control->variables()->get("connection_seed")));
-  m_variables.insert("directory",        new utils::VariableAny(control->variables()->get("directory")));
+  m_variables.insert("connection_leech", new utils::VariableAny(control->variable()->get("connection_leech")));
+  m_variables.insert("connection_seed",  new utils::VariableAny(control->variable()->get("connection_seed")));
+  m_variables.insert("directory",        new utils::VariableAny(control->variable()->get("directory")));
   m_variables.insert("tied_to_file",     new utils::VariableBool(false));
 }
 
@@ -101,7 +104,7 @@ DownloadFactory::receive_load() {
   if (std::strncmp(m_uri.c_str(), "http://", 7) == 0) {
     // Http handling here.
     m_stream = new std::stringstream;
-    HttpQueue::iterator itr = m_manager->http_queue().insert(m_uri, m_stream);
+    HttpQueue::iterator itr = m_manager->http_queue()->insert(m_uri, m_stream);
 
     (*itr)->signal_done().slots().push_front(sigc::mem_fun(*this, &DownloadFactory::receive_loaded));
     (*itr)->signal_failed().slots().push_front(sigc::mem_fun(*this, &DownloadFactory::receive_failed));
@@ -137,78 +140,76 @@ DownloadFactory::receive_commit() {
 void
 DownloadFactory::receive_success() {
   if (m_stream == NULL)
-    throw torrent::client_error("DownloadFactory::receive_success() called on an object with m_stream == NULL");
+    throw torrent::client_error("DownloadFactory::receive_success() called on an object with m_stream == NULL.");
 
-  Manager::DListItr itr = m_manager->insert(m_stream, m_printLog);
+  Download* download = m_manager->download_list()->create(m_stream, m_printLog);
 
-  if (itr == m_manager->download_list().end()) {
+  if (download == NULL) {
     // core::Manager should already have added the error message to
     // the log.
     m_slotFinished();
     return;
   }
 
-  torrent::Bencode& root = (*itr)->get_bencode();
+  torrent::Object* root = download->bencode();
 
   if (!m_session) {
     // We only allow session torrents to keep their
     // 'rtorrent/libtorrent' sections.
-    root.erase_key("rtorrent");
-    root.erase_key("libtorrent");
+    root->erase_key("rtorrent");
+    root->erase_key("libtorrent");
   }
 
-  if (!root.has_key("rtorrent") ||
-      !root.get_key("rtorrent").is_map())
-    root.insert_key("rtorrent", torrent::Bencode(torrent::Bencode::TYPE_MAP));
+  if (!root->has_key_map("rtorrent"))
+    root->insert_key("rtorrent", torrent::Object(torrent::Object::TYPE_MAP));
     
-  torrent::Bencode& rtorrent = root.get_key("rtorrent");
+  torrent::Object* rtorrent = &root->get_key("rtorrent");
 
-  if (!rtorrent.has_key("state") ||
-      !rtorrent.get_key("state").is_string() ||
-      (rtorrent.get_key("state").as_string() != "stopped" &&
-       rtorrent.get_key("state").as_string() != "started"))
-    rtorrent.insert_key("state", "stopped");
-  
-  if (!rtorrent.has_key("tied_to_file") ||
-      !rtorrent.get_key("tied_to_file").is_string())
-    rtorrent.insert_key("tied_to_file", std::string());
-
-  if (rtorrent.has_key("priority") &&
-      rtorrent.get_key("priority").is_value())
-    (*itr)->variables()->set("priority", rtorrent.get_key("priority").as_value() % 4);
-  else
-    (*itr)->variables()->set("priority", (int64_t)2);
+  initialize_rtorrent(download, rtorrent);
 
   // Move to 'rtorrent'.
-  (*itr)->variables()->set("connection_leech", m_variables.get("connection_leech"));
-  (*itr)->variables()->set("connection_seed",  m_variables.get("connection_seed"));
-  (*itr)->variables()->set("min_peers",        control->variables()->get("min_peers"));
-  (*itr)->variables()->set("max_peers",        control->variables()->get("max_peers"));
-  (*itr)->variables()->set("max_uploads",      control->variables()->get("max_uploads"));
+  download->variable()->set("connection_leech", m_variables.get("connection_leech"));
+  download->variable()->set("connection_seed",  m_variables.get("connection_seed"));
+  download->variable()->set("min_peers",        control->variable()->get("min_peers"));
+  download->variable()->set("max_peers",        control->variable()->get("max_peers"));
+  download->variable()->set("max_uploads",      control->variable()->get("max_uploads"));
 
-  if (control->variables()->get("use_udp_trackers").as_string() == "no")
-    (*itr)->enable_udp_trackers(false);
+  if (!control->variable()->get_value("use_udp_trackers"))
+    download->enable_udp_trackers(false);
 
+  if (!rtorrent->has_key_string("directory"))
+    download->variable()->set("directory", m_variables.get("directory"));
+  else
+    download->variable()->set("directory", rtorrent->get_key("directory"));
+
+  if (!m_session && m_variables.get("tied_to_file").as_value())
+    download->variable()->set("tied_to_file", m_uri);
+
+  // The action of inserting might cause the torrent to be
+  // opened/started or such. Figure out a nicer way of handling this.
+  if (m_manager->download_list()->insert(download) == m_manager->download_list()->end()) {
+    // ATM doesn't really ever get here.
+    delete download;
+
+    m_slotFinished();
+    return;
+  }
+
+  // When a download scheduler is implemented, this is handled by the
+  // above insertion into download list.
   if (m_session) {
-    if (!rtorrent.has_key("directory") ||
-	!rtorrent.get_key("directory").is_string())
-      (*itr)->variables()->set("directory", m_variables.get("directory"));
-    else
-      (*itr)->variables()->set("directory", rtorrent.get_key("directory"));
-
-    if ((*itr)->variables()->get_string("state") == "started")
-      m_manager->start(*itr, m_printLog);
+    // This torrent was queued for hashing or hashing when the session
+    // file was saved. Or it was in a started state.
+    if (download->variable()->get_value("hashing") != Download::variable_hashing_stopped ||
+	download->variable()->get_value("state") != 0)
+      m_manager->download_list()->resume(download);
 
   } else {
-    (*itr)->variables()->set("directory", m_variables.get("directory"));
-
-    if (m_variables.get("tied_to_file").as_value())
-      (*itr)->variables()->set("tied_to_file", m_uri);
-
+    // Use the state thingie here, move below.
     if (m_start)
-      m_manager->start(*itr, m_printLog);
+      m_manager->download_list()->start(download);
 
-    m_manager->download_store().save(*itr);
+    m_manager->download_store()->save(download);
   }
 
   m_slotFinished();
@@ -228,4 +229,41 @@ DownloadFactory::receive_failed(const std::string& msg) {
   m_slotFinished();
 }
 
+void
+DownloadFactory::initialize_rtorrent(Download* download, torrent::Object* rtorrent) {
+  if (!rtorrent->has_key_value("state") || rtorrent->get_key("state").as_value() > 1) {
+    rtorrent->insert_key("state", (int64_t)m_start);
+    rtorrent->insert_key("state_changed", cachedTime.seconds());
+
+  } else if (!rtorrent->has_key_value("state_changed") ||
+	     rtorrent->get_key("state_changed").as_value() > cachedTime.seconds() || rtorrent->get_key("state_changed").as_value() == 0) {
+    rtorrent->insert_key("state_changed", cachedTime.seconds());
+  }
+
+  if (!rtorrent->has_key_value("complete"))
+    rtorrent->insert_key("complete", (int64_t)0);
+
+  if (!rtorrent->has_key_value("hashing"))
+    rtorrent->insert_key("hashing", Download::variable_hashing_stopped);
+
+  if (!rtorrent->has_key_string("tied_to_file"))
+    rtorrent->insert_key("tied_to_file", std::string());
+
+  if (rtorrent->has_key_value("priority"))
+    download->variable()->set("priority", rtorrent->get_key("priority").as_value() % 4);
+  else
+    download->variable()->set("priority", (int64_t)2);
+
+  if (rtorrent->has_key_value("key")) {
+    download->tracker_list()->set_key(rtorrent->get_key("key").as_value());
+
+  } else {
+    download->tracker_list()->set_key(rand() % (std::numeric_limits<uint32_t>::max() - 1) + 1);
+    rtorrent->insert_key("key", download->tracker_list()->key());
+  }
+
+  if (rtorrent->has_key_value("total_uploaded"))
+    download->download()->up_rate()->set_total(rtorrent->get_key("total_uploaded").as_value());
+}
+
 }
