@@ -41,11 +41,14 @@
 #include <fstream>
 #include <stdio.h>
 #include <unistd.h>
+#include <rak/error_number.h>
 #include <rak/path.h>
 #include <rak/string_manip.h>
-#include <torrent/bencode.h>
+#include <torrent/object.h>
 #include <torrent/exceptions.h>
 #include <torrent/torrent.h>
+#include <torrent/rate.h>
+#include <torrent/object_stream.h>
 
 #include "download.h"
 #include "download_store.h"
@@ -66,7 +69,10 @@ DownloadStore::enable(bool lock) {
     m_lockfile.set_path(std::string());
 
   if (!m_lockfile.try_lock())
-    throw torrent::input_error("Could not lock session directory: \"" + m_path + "\", held by \"" + m_lockfile.locked_by_as_string() + "\".");
+    if (rak::error_number::current().is_bad_path())
+      throw torrent::input_error("Could not lock session directory: \"" + m_path + "\", " + rak::error_number::current().c_str());
+    else
+      throw torrent::input_error("Could not lock session directory: \"" + m_path + "\", held by \"" + m_lockfile.locked_by_as_string() + "\".");
 }
 
 void
@@ -98,7 +104,10 @@ DownloadStore::save(Download* d) {
   if (!f.is_open())
     return;
 
-  f << d->get_bencode();
+  // Move this somewhere else.
+  d->bencode()->get_key("rtorrent").insert_key("total_uploaded", d->download()->up_rate()->total());
+
+  f << *d->bencode();
 
   if (!f.good())
     return;
@@ -108,7 +117,7 @@ DownloadStore::save(Download* d) {
   // Test the new file, to ensure it is a valid bencode string.
   f.open((create_filename(d) + ".new").c_str(), std::ios::in);
 
-  torrent::Bencode tmp;
+  torrent::Object tmp;
   f >> tmp;
 
   if (!f.good())
@@ -157,7 +166,7 @@ DownloadStore::is_correct_format(std::string f) {
 
 std::string
 DownloadStore::create_filename(Download* d) {
-  return m_path + rak::transform_hex(d->get_hash()) + ".torrent";
+  return m_path + rak::transform_hex(d->info_hash()) + ".torrent";
 }
 
 }
