@@ -40,15 +40,18 @@
 #include <cstdio>
 #include <cstring>
 #include <fstream>
-#include <istream>
 #include <unistd.h>
 #include <sys/select.h>
+#include <rak/address_info.h>
+#include <rak/error_number.h>
 #include <rak/regex.h>
 #include <rak/string_manip.h>
 #include <sigc++/bind.h>
 #include <sigc++/hide.h>
-#include <torrent/bencode.h>
+#include <torrent/object.h>
+#include <torrent/connection_manager.h>
 #include <torrent/exceptions.h>
+#include <torrent/tracker_list.h>
 
 #include "utils/variable_map.h"
 
@@ -56,26 +59,53 @@
 #include "curl_get.h"
 #include "download.h"
 #include "download_factory.h"
+#include "download_store.h"
+#include "http_queue.h"
 #include "manager.h"
 #include "poll_manager_epoll.h"
 #include "poll_manager_select.h"
+#include "view.h"
 
 namespace core {
 
 static void
-connect_signal_network_log(Download* d, torrent::Download::SlotString s) {
-  d->get_download().signal_network_log(s);
+connect_signal_network_log(Download* d, torrent::Download::slot_string_type s) {
+  d->download()->signal_network_log(s);
 }
 
 static void
-connect_signal_storage_log(Download* d, torrent::Download::SlotString s) {
-  d->get_download().signal_storage_error(s);
+connect_signal_storage_log(Download* d, torrent::Download::slot_string_type s) {
+  d->download()->signal_storage_error(s);
+}
+
+// Need a proper logging class for this.
+static void
+connect_signal_tracker_dump(Download* d, torrent::Download::slot_dump_type s) {
+  if (!control->variable()->get_string("tracker_dump").empty())
+    d->download()->signal_tracker_dump(s);
+}
+
+static void
+receive_tracker_dump(const std::string& url, const char* data, size_t size) {
+  const std::string& filename = control->variable()->get_string("tracker_dump");
+
+  if (filename.empty())
+    return;
+
+  std::fstream fstr(filename.c_str(), std::ios::out | std::ios::app);
+
+  if (!fstr.is_open())
+    return;
+
+  fstr << "url: " << url << std::endl << "---" << std::endl;
+  fstr.write(data, size);
+  fstr << std::endl <<"---" << std::endl;
 }
 
 // Hmm... find some better place for all this.
 static void
 delete_tied(Download* d) {
-  const std::string tie = d->variables()->get("tied_to_file").as_string();
+  const std::string tie = d->variable()->get("tied_to_file").as_string();
 
   // This should be configurable, need to wait for the variable
   // thingie to be implemented.
@@ -84,9 +114,32 @@ delete_tied(Download* d) {
 }
 
 Manager::Manager() :
+  m_hashingView(NULL),
+
   m_pollManager(NULL),
   m_portFirst(6890),
   m_portLast(6999) {
+
+  m_downloadStore = new DownloadStore();
+  m_downloadList = new DownloadList();
+
+  m_httpQueue = new HttpQueue();
+}
+
+Manager::~Manager() {
+  delete m_downloadList;
+
+  delete m_downloadStore;
+  delete m_httpQueue;
+}
+
+void
+Manager::set_hashing_view(View* v) {
+  if (v == NULL || m_hashingView != NULL)
+    throw torrent::client_error("Manager::set_hashing_view(...) received NULL or is already set.");
+
+  m_hashingView = v;
+  v->signal_changed().connect(sigc::mem_fun(this, &Manager::receive_hashing_changed));
 }
 
 void
@@ -106,34 +159,17 @@ Manager::initialize_first() {
 void
 Manager::initialize_second() {
   torrent::Http::set_factory(m_pollManager->get_http_stack()->get_http_factory());
-  m_httpQueue.slot_factory(m_pollManager->get_http_stack()->get_http_factory());
+  m_httpQueue->slot_factory(m_pollManager->get_http_stack()->get_http_factory());
 
   CurlStack::global_init();
 
   // Register slots to be called when a download is inserted/erased,
   // opened or closed.
-  m_downloadList.slot_map_insert()["1_connect_network_log"] = sigc::bind(sigc::ptr_fun(&connect_signal_network_log), sigc::mem_fun(m_logComplete, &Log::push_front));
-  m_downloadList.slot_map_insert()["1_connect_storage_log"] = sigc::bind(sigc::ptr_fun(&connect_signal_storage_log), sigc::mem_fun(m_logComplete, &Log::push_front));
+  m_downloadList->slot_map_insert()["1_connect_network_log"]  = sigc::bind(sigc::ptr_fun(&connect_signal_network_log), sigc::mem_fun(m_logComplete, &Log::push_front));
+  m_downloadList->slot_map_insert()["1_connect_storage_log"]  = sigc::bind(sigc::ptr_fun(&connect_signal_storage_log), sigc::mem_fun(m_logComplete, &Log::push_front));
+  m_downloadList->slot_map_insert()["1_connect_tracker_dump"] = sigc::bind(sigc::ptr_fun(&connect_signal_tracker_dump), sigc::ptr_fun(&receive_tracker_dump));
 
-  m_downloadList.slot_map_erase()["1_hash_queue_remove"]    = sigc::mem_fun(m_hashQueue, &HashQueue::remove);
-  m_downloadList.slot_map_erase()["1_store_remove"]         = sigc::mem_fun(m_downloadStore, &DownloadStore::remove);
-  m_downloadList.slot_map_erase()["1_delete_tied"]          = sigc::ptr_fun(&delete_tied);
-
-  m_downloadList.slot_map_open()["1_download_open"]         = sigc::mem_fun(&Download::call<void, &torrent::Download::open>);
-
-  // Currently does not call stop, might want to add a function that
-  // checks if we're running, and if so stop?
-  m_downloadList.slot_map_close()["1_hash_queue_remove"]    = sigc::mem_fun(m_hashQueue, &HashQueue::remove);
-  m_downloadList.slot_map_close()["2_download_close"]       = sigc::mem_fun(&Download::call<void, &torrent::Download::close>);
-
-  m_downloadList.slot_map_start()["1_download_start"]       = sigc::mem_fun(&Download::start);
-
-  m_downloadList.slot_map_stop()["1_download_stop"]         = sigc::mem_fun(&Download::call<void, &torrent::Download::stop>);
-  m_downloadList.slot_map_stop()["2_hash_resume_save"]      = sigc::mem_fun(&Download::call<void, &torrent::Download::hash_resume_save>);
-  m_downloadList.slot_map_stop()["3_store_save"]            = sigc::mem_fun(m_downloadStore, &DownloadStore::save);
-
-  m_downloadList.slot_map_finished()["1_download_done"]     = sigc::mem_fun(*this, &Manager::receive_download_done);
-  m_downloadList.slot_map_finished()["2_receive_finished"]  = sigc::mem_fun(&Download::receive_finished);
+  m_downloadList->slot_map_erase()["1_delete_tied"] = sigc::ptr_fun(&delete_tied);
 }
 
 void
@@ -141,6 +177,8 @@ Manager::cleanup() {
   // Need to disconnect log signals? Not really since we won't receive
   // any more.
 
+  m_downloadList->clear();
+
   torrent::cleanup();
   CurlStack::global_cleanup();
 
@@ -149,156 +187,97 @@ Manager::cleanup() {
 
 void
 Manager::shutdown(bool force) {
+  // This doesn't trigger a compiler error on gcc-3.4.5 for some reason.
+//   if (!force)
+//     std::for_each(m_downloadList->begin(), m_downloadList->end(), std::bind1st(std::mem_fun(&DownloadList::pause), &m_downloadList));
+//   else
+//     std::for_each(m_downloadList->begin(), m_downloadList->end(), std::bind1st(std::mem_fun(&DownloadList::close), &m_downloadList));
+
   if (!force)
-    std::for_each(m_downloadList.begin(), m_downloadList.end(),
-		  std::bind1st(std::mem_fun(&DownloadList::stop), &m_downloadList));
+    std::for_each(m_downloadList->begin(), m_downloadList->end(), std::bind1st(std::mem_fun(&DownloadList::pause), m_downloadList));
   else
-    std::for_each(m_downloadList.begin(), m_downloadList.end(),
-		  std::bind1st(std::mem_fun(&DownloadList::close), &m_downloadList));
-}
-
-Manager::DListItr
-Manager::insert(std::istream* s, bool printLog) {
-  try {
-    return m_downloadList.insert(s);
-
-  } catch (torrent::local_error& e) {
-    if (printLog) {
-      m_logImportant.push_front(e.what());
-      m_logComplete.push_front(e.what());
-    }
-
-    return m_downloadList.end();
-  }
-}
-
-Manager::DListItr
-Manager::erase(DListItr itr) {
-  if ((*itr)->get_download().is_active())
-    throw std::logic_error("core::Manager::erase(...) called on an active download");
-
-//   if (!(*itr)->get_download().is_open())
-//     throw std::logic_error("core::Manager::erase(...) called on an closed download");
-
-  return m_downloadList.erase(itr);
-}  
-
-void
-Manager::start(Download* d, bool printLog) {
-  try {
-    d->variables()->set("state", "started");
-
-    if (d->get_download().is_active())
-      return;
-
-    if (!d->get_download().is_open())
-      m_downloadList.open(d);
-
-    if (d->get_download().is_hash_checked())
-      m_downloadList.start(d);
-    else
-      // This can cause infinit loops?
-      m_hashQueue.insert(d, sigc::bind(sigc::mem_fun(m_downloadList, &DownloadList::start), d));
-
-  } catch (torrent::local_error& e) {
-    if (printLog) {
-      m_logImportant.push_front(e.what());
-      m_logComplete.push_front(e.what());
-    }
-  }
-}
-
-void
-Manager::stop(Download* d) {
-  try {
-    d->variables()->set("state", "stopped");
-
-    m_downloadList.stop(d);
-
-  } catch (torrent::local_error& e) {
-    m_logImportant.push_front(e.what());
-    m_logComplete.push_front(e.what());
-  }
-}
-
-void
-Manager::check_hash(Download* d) {
-  bool restart = d->get_download().is_active();
-
-  try {
-    prepare_hash_check(d);
-
-    if (restart)
-      m_hashQueue.insert(d, sigc::bind(sigc::mem_fun(m_downloadList, &DownloadList::start), d));
-    else
-      m_hashQueue.insert(d, sigc::slot0<void>());
-
-  } catch (torrent::local_error& e) {
-    m_logImportant.push_front(e.what());
-    m_logComplete.push_front(e.what());
-  }
-}  
-
-void
-Manager::receive_download_done(Download* d) {
-  if (control->variables()->get("check_hash").as_string() == "yes") {
-    // Start the hash checking, send completed to tracker after
-    // finishing.
-    prepare_hash_check(d);
-
-    // TODO: Need to restart the torrent.
-    m_hashQueue.insert(d, sigc::bind(sigc::mem_fun(*this, &Manager::receive_download_done_hash_checked), d));
-
-  } else {
-    receive_download_done_hash_checked(d);
-  }
+    std::for_each(m_downloadList->begin(), m_downloadList->end(), std::bind1st(std::mem_fun(&DownloadList::close), m_downloadList));
 }
 
 void
 Manager::listen_open() {
+  if (!control->variable()->get_value("port_open"))
+    return;
+
   if (m_portFirst > m_portLast)
     throw torrent::input_error("Invalid port range for listening");
 
-  if (control->variables()->get("port_random").as_string() == "yes") {
+  if (control->variable()->get_value("port_random")) {
     int boundary = m_portFirst + random() % (m_portLast - m_portFirst + 1);
 
-    if (!torrent::listen_open(boundary, m_portLast) &&
-	!torrent::listen_open(m_portFirst, boundary))
-      throw torrent::input_error("Could not open/bind a port for listening.");
+    if (torrent::connection_manager()->listen_open(boundary, m_portLast) ||
+	torrent::connection_manager()->listen_open(m_portFirst, boundary))
+      return;
 
   } else {
-    if (!torrent::listen_open(m_portFirst, m_portLast))
-      throw torrent::input_error("Could not open/bind a port for listening.");
-
+    if (torrent::connection_manager()->listen_open(m_portFirst, m_portLast))
+      return;
   }
+
+  throw torrent::input_error("Could not open/bind a port for listening: " + std::string(rak::error_number::current().c_str()));
+}
+
+std::string
+Manager::bind_address() const {
+  return rak::socket_address::cast_from(torrent::connection_manager()->bind_address())->address_str();
 }
 
 void
-Manager::bind(const std::string& addr) {
-  if (torrent::listen_port() != 0) {
-    torrent::listen_close();
-    torrent::set_bind_address(addr);
-    listen_open();
+Manager::set_bind_address(const std::string& addr) {
+  int err;
+  rak::address_info* ai;
 
-  } else {
-    torrent::set_bind_address(addr);
+  if ((err = rak::address_info::get_address_info(addr.c_str(), PF_INET, SOCK_STREAM, &ai)) != 0)
+    throw torrent::input_error("Could not set bind address: " + std::string(rak::address_info::strerror(err)) + ".");
+  
+  try {
+
+    if (torrent::connection_manager()->listen_port() != 0) {
+      torrent::connection_manager()->listen_close();
+      torrent::connection_manager()->set_bind_address(ai->address()->c_sockaddr());
+      listen_open();
+
+    } else {
+      torrent::connection_manager()->set_bind_address(ai->address()->c_sockaddr());
+    }
+
+    m_pollManager->get_http_stack()->set_bind_address(!ai->address()->is_address_any() ? ai->address()->address_str() : std::string());
+
+    rak::address_info::free_address_info(ai);
+
+  } catch (torrent::input_error& e) {
+    rak::address_info::free_address_info(ai);
+    throw e;
   }
+}
 
-  m_pollManager->get_http_stack()->set_bind_address(torrent::bind_address());
+std::string
+Manager::local_address() const {
+  return rak::socket_address::cast_from(torrent::connection_manager()->local_address())->address_str();
 }
 
 void
-Manager::prepare_hash_check(Download* d) {
-  m_downloadList.close(d);
-  d->get_download().hash_resume_clear();
-  m_downloadList.open(d);
+Manager::set_local_address(const std::string& addr) {
+  int err;
+  rak::address_info* ai;
 
-  if (d->get_download().is_hash_checking() ||
-      d->get_download().is_hash_checked())
-    throw std::logic_error("Manager::check_hash(...) closed the torrent but is_hash_check{ing,ed}() == true");
+  if ((err = rak::address_info::get_address_info(addr.c_str(), PF_INET, SOCK_STREAM, &ai)) != 0)
+    throw torrent::input_error("Could not set local address: " + std::string(rak::address_info::strerror(err)) + ".");
+  
+  try {
 
-  if (m_hashQueue.find(d) != m_hashQueue.end())
-    throw std::logic_error("Manager::check_hash(...) closed the torrent but it was found in m_hashQueue");
+    torrent::connection_manager()->set_local_address(ai->address()->c_sockaddr());
+    rak::address_info::free_address_info(ai);
+
+  } catch (torrent::input_error& e) {
+    rak::address_info::free_address_info(ai);
+    throw e;
+  }
 }
 
 void
@@ -307,25 +286,12 @@ Manager::receive_http_failed(std::string msg) {
   m_logComplete.push_front("Http download error: \"" + msg + "\"");
 }
 
-void
-Manager::receive_download_done_hash_checked(Download* d) {
-  if (!d->get_download().is_active())
-    m_downloadList.start(d);
-
-  if (control->variables()->get_string("session_on_completion") == "yes")
-    m_downloadStore.save(d);
-
-  // Don't send if we did a hash check and found incompelete chunks.
-  if (d->is_done())
-    d->get_download().tracker_send_completed();
-}
-
 void
 Manager::try_create_download(const std::string& uri, bool start, bool printLog, bool tied) {
   // Adding download.
   DownloadFactory* f = new DownloadFactory(uri, this);
 
-  f->variables()->set("tied_to_file", tied ? "yes" : "no");
+  f->variable()->set("tied_to_file", tied ? "yes" : "no");
 
   f->set_start(start);
   f->set_print_log(printLog);
@@ -389,8 +355,8 @@ Manager::try_create_download_expand(const std::string& uri, bool start, bool pri
 
   if (tied)
     for (std::vector<std::string>::iterator itr = paths.begin(); itr != paths.end(); )
-      if (std::find_if(m_downloadList.begin(), m_downloadList.end(),
-		       rak::equal(*itr, rak::bind2nd(std::mem_fun(&Download::variable_string), "tied_to_file"))) != m_downloadList.end())
+      if (std::find_if(m_downloadList->begin(), m_downloadList->end(),
+		       rak::equal(*itr, rak::bind2nd(std::mem_fun(&Download::variable_string), "tied_to_file"))) != m_downloadList->end())
 	itr = paths.erase(itr);
       else
 	itr++;
@@ -403,4 +369,35 @@ Manager::try_create_download_expand(const std::string& uri, bool start, bool pri
     try_create_download(uri, start, printLog, tied);
 }
 
+// DownloadList's hashing related functions don't actually start the
+// hashing, it only reacts to events. This functions checks the
+// hashing view and starts hashing if nessesary.
+void
+Manager::receive_hashing_changed() {
+  if (m_hashingView->empty_visible() ||
+      std::find_if(m_hashingView->begin_visible(), m_hashingView->end_visible(), std::mem_fun(&Download::is_hash_checking)) != m_hashingView->end_visible())
+    return;
+
+  for (View::iterator itr = m_hashingView->begin_visible(), last = m_hashingView->end_visible(); itr != last; ++itr) {
+
+    try {
+
+      if ((*itr)->is_hash_checked())
+	throw torrent::client_error("core::Manager::receive_hashing_changed() hash already checked.");
+  
+      if ((*itr)->is_hash_failed())
+	continue;
+
+      m_downloadList->open_throw(*itr);
+      (*itr)->download()->hash_check();
+
+      return;
+
+    } catch (torrent::local_error& e) {
+      (*itr)->set_hash_failed(true);
+      push_log(e.what());
+    }
+  }
+}
+
 }
