@@ -40,9 +40,6 @@
 #include <iosfwd>
 
 #include "download_list.h"
-#include "download_store.h"
-#include "hash_queue.h"
-#include "http_queue.h"
 #include "poll_manager.h"
 #include "log.h"
 
@@ -52,6 +49,10 @@ namespace torrent {
 
 namespace core {
 
+class DownloadStore;
+class HttpQueue;
+class View;
+
 class Manager {
 public:
   typedef DownloadList::iterator                    DListItr;
@@ -59,11 +60,15 @@ public:
   typedef sigc::slot0<void>                         SlotFailed;
 
   Manager();
+  ~Manager();
 
-  DownloadList&       download_list()                 { return m_downloadList; }
-  DownloadStore&      download_store()                { return m_downloadStore; }
-  HashQueue&          hash_queue()                    { return m_hashQueue; }
-  HttpQueue&          http_queue()                    { return m_httpQueue; }
+  DownloadList*       download_list()                     { return m_downloadList; }
+  DownloadStore*      download_store()                    { return m_downloadStore; }
+
+  HttpQueue*          http_queue()                        { return m_httpQueue; }
+
+  View*               hashing_view()                      { return m_hashingView; }
+  void                set_hashing_view(View* v);
 
   PollManager*        get_poll_manager()                  { return m_pollManager; }
   Log&                get_log_important()                 { return m_logImportant; }
@@ -78,18 +83,14 @@ public:
 
   void                listen_open();
 
-  void                bind(const std::string& addr);
+  std::string         bind_address() const;
+  void                set_bind_address(const std::string& addr);
+
+  std::string         local_address() const;
+  void                set_local_address(const std::string& addr);
 
   void                shutdown(bool force);
 
-  DListItr            insert(std::istream* s, bool printLog = true);
-  DListItr            erase(DListItr itr);
-
-  void                start(Download* d, bool printLog = true);
-  void                stop(Download* d);
-
-  void                check_hash(Download* d);
-
   void                push_log(const std::string& msg)    { m_logImportant.push_front(msg); m_logComplete.push_front(msg); }
 
   // Temporary, find a better place for this.
@@ -102,17 +103,15 @@ private:
 
   void                initialize_bencode(Download* d);
 
-  void                prepare_hash_check(Download* d);
-
   void                receive_http_failed(std::string msg);
-  void                receive_download_done_hash_checked(Download* d);
-  void                receive_download_inserted(Download* d);
-  void                receive_download_done(Download* d);
 
-  DownloadList        m_downloadList;
-  DownloadStore       m_downloadStore;
-  HashQueue           m_hashQueue;
-  HttpQueue           m_httpQueue;
+  void                receive_hashing_changed();
+
+  DownloadList*       m_downloadList;
+  DownloadStore*      m_downloadStore;
+  HttpQueue*          m_httpQueue;
+
+  View*               m_hashingView;
 
   PollManager*        m_pollManager;
   Log                 m_logImportant;
