@@ -1,5 +1,5 @@
 // rTorrent - BitTorrent client
-// Copyright (C) 2005, Jari Sundell
+// Copyright (C) 2005-2006, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -47,7 +47,8 @@
 
 namespace display {
 
-Manager::Manager() {
+Manager::Manager() :
+  m_forceRedraw(false) {
   m_taskUpdate.set_slot(rak::mem_fn(this, &Manager::receive_update));
 }
 
@@ -55,6 +56,12 @@ Manager::~Manager() {
   priority_queue_erase(&taskScheduler, &m_taskUpdate);
 }
 
+void
+Manager::force_redraw() {
+  m_forceRedraw = true;
+}
+
+
 Manager::iterator
 Manager::insert(iterator pos, Window* w) {
   return Base::insert(pos, w);
@@ -131,6 +138,14 @@ Manager::adjust_layout() {
 
 void
 Manager::receive_update() {
+  if (m_forceRedraw) {
+    m_forceRedraw = false;
+
+    display::Canvas::resize_term(display::Canvas::term_size());
+    Canvas::redraw_std();
+    adjust_layout();
+  }
+
   Canvas::refresh_std();
 
   rak::priority_queue_perform(&m_scheduler, cachedTime);
