@@ -37,12 +37,15 @@
 #include "config.h"
 
 #include <stdexcept>
+#include <rak/socket_address.h>
 #include <torrent/rate.h>
+#include <torrent/piece.h>
 
 #include "core/download.h"
 #include "rak/algorithm.h"
 
 #include "canvas.h"
+#include "client_info.h"
 #include "window_peer_list.h"
 
 namespace display {
@@ -62,7 +65,7 @@ WindowPeerList::redraw() {
   int x = 2;
   int y = 0;
 
-  m_canvas->print(x, y, "DNS");     x += 16;
+  m_canvas->print(x, y, "IP");     x += 16;
   m_canvas->print(x, y, "UP");      x += 7;
   m_canvas->print(x, y, "DOWN");    x += 7;
   m_canvas->print(x, y, "PEER");    x += 7;
@@ -84,7 +87,7 @@ WindowPeerList::redraw() {
 					   m_list->end(),
 					   m_canvas->get_height() - y);
 
-  if (m_download->get_download().chunks_total() <= 0)
+  if (m_download->download()->chunks_total() <= 0)
     throw std::logic_error("WindowPeerList::redraw() m_slotChunksTotal() returned invalid value");
 
   while (range.first != range.second) {
@@ -94,7 +97,7 @@ WindowPeerList::redraw() {
 
     m_canvas->print(x, y, "%c %s",
 		    range.first == *m_focus ? '*' : ' ',
-		    p.address().c_str());
+		    rak::socket_address::cast_from(p.address())->address_str().c_str());
     x += 18;
 
     m_canvas->print(x, y, "%.1f", (double)p.up_rate()->rate() / 1024); x += 7;
@@ -118,13 +121,20 @@ WindowPeerList::redraw() {
     x += 6;
 
     if (p.incoming_queue_size())
-      m_canvas->print(x, y, "%i", p.incoming_index(0));
+      m_canvas->print(x, y, "%i", p.incoming_queue(0)->index());
 
     x += 6;
 
     if (p.is_snubbed())
       m_canvas->print(x, y, "*");
 
+    x += 5;
+
+    char buf[128];
+    control->client_info()->print(buf, buf + 128, p.id().c_str());
+
+    m_canvas->print(x, y, "%s", buf);
+
     ++y;
     ++range.first;
   }
@@ -132,7 +142,7 @@ WindowPeerList::redraw() {
 
 int
 WindowPeerList::done_percentage(torrent::Peer& p) {
-  int chunks = m_download->get_download().chunks_total();
+  int chunks = m_download->download()->chunks_total();
 
   return chunks ? (100 * p.chunks_done()) / chunks : 0;
 }
