@@ -1,5 +1,5 @@
 // rTorrent - BitTorrent client
-// Copyright (C) 2005, Jari Sundell
+// Copyright (C) 2005-2006, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -39,6 +39,7 @@
 #include <functional>
 #include <rak/algorithm.h>
 #include <rak/file_stat.h>
+#include <rak/path.h>
 
 #include "path_input.h"
 
@@ -70,7 +71,7 @@ struct _transform_filename {
   void operator () (std::string& filename) {
     rak::file_stat fs;
 
-    if (!fs.update((m_base + filename)))
+    if (!fs.update(rak::path_expand(m_base + filename)))
       return;
 
     else if (fs.is_directory())
