@@ -36,7 +36,11 @@
 
 #include "config.h"
 
+#include <algorithm>
+#include <cstring>
 #include <cstdio>
+#include <functional>
+#include <getopt.h>
 #include <stdexcept>
 #include <unistd.h>
 #include <sigc++/bind.h>
@@ -73,6 +77,7 @@ OptionParser::process(int argc, char** argv) {
   int c;
   std::string optString = create_optstring();
 
+  optind = 0;
   opterr = 0;
 
   while ((c = getopt(argc, argv, optString.c_str())) != -1)
@@ -84,6 +89,13 @@ OptionParser::process(int argc, char** argv) {
   return optind;
 }
 
+bool
+OptionParser::has_flag(char flag, int argc, char** argv) {
+  char options[3] = { '-', flag, '\0' };
+
+  return std::find_if(argv, argv + argc, std::not1(std::bind1st(std::ptr_fun(&std::strcmp), options))) != argv + argc;
+}
+
 std::string
 OptionParser::create_optstring() {
   std::string s;
