@@ -74,6 +74,7 @@ public:
     DISPLAY_PEER_INFO,
     DISPLAY_FILE_LIST,
     DISPLAY_TRACKER_LIST,
+    DISPLAY_CHUNKS_SEEN,
     DISPLAY_MAX_SIZE
   } Display;
 
@@ -98,6 +99,8 @@ private:
   void                receive_next();
   void                receive_prev();
 
+  void                receive_disconnect_peer();
+
   void                receive_peer_connected(torrent::Peer p);
   void                receive_peer_disconnected(torrent::Peer p);
 
