@@ -45,7 +45,10 @@
 #include <torrent/torrent.h>
 
 #include "core/download.h"
+#include "core/download_list.h"
 #include "core/manager.h"
+#include "core/view.h"
+#include "core/view_manager.h"
 
 #include "input/bindings.h"
 #include "input/manager.h"
@@ -73,19 +76,22 @@ DownloadList::DownloadList(Control* c) :
   m_window(c->display()->end()),
 
   m_windowTitle(new WTitle("rTorrent " VERSION " - libTorrent " + std::string(torrent::version()))),
-  m_windowHttpQueue(new WHttp(&c->core()->http_queue())),
+  m_windowHttpQueue(new WHttp(c->core()->http_queue())),
 
   m_uiDownload(NULL),
 
-  m_downloadList(&c->core()->download_list()),
-
   m_control(c),
   m_bindings(new input::Bindings)
 {
-  m_uiArray[DISPLAY_DOWNLOAD_LIST] = new ElementDownloadList(&m_downloadList);
+  m_uiArray[DISPLAY_DOWNLOAD_LIST] = new ElementDownloadList();
   m_uiArray[DISPLAY_LOG]           = new ElementLogComplete(&m_control->core()->get_log_complete());
   m_windowLog                      = new WLog(&m_control->core()->get_log_important());
 
+  receive_change_view("main");
+
+  if (m_view == NULL)
+    throw torrent::client_error("View \"main\" must be present to initialize the main display.");
+
   m_taskUpdate.set_slot(rak::mem_fn(this, &DownloadList::task_update)),
 
   setup_keys();
@@ -124,7 +130,7 @@ DownloadList::activate() {
 
   m_control->input()->push_front(m_bindings);
 
-  m_control->core()->download_list().slot_map_erase()["0_download_list"] = sigc::mem_fun(this, &DownloadList::receive_download_erased);
+  m_control->core()->download_list()->slot_map_erase()["0_download_list"] = sigc::mem_fun(this, &DownloadList::receive_download_erased);
 
   activate_display(DISPLAY_DOWNLOAD_LIST);
 }
@@ -179,36 +185,50 @@ DownloadList::disable_display() {
 
 void
 DownloadList::receive_next() {
-  m_downloadList.inc_focus();
+  m_view->next_focus();
+  m_view->set_last_changed();
 }
 
 void
 DownloadList::receive_prev() {
-  m_downloadList.dec_focus();
+  m_view->prev_focus();
+  m_view->set_last_changed();
 }
 
 void
 DownloadList::receive_start_download() {
-  if (m_downloadList.get_focus() == m_downloadList.end())
+  if (m_view->focus() == m_view->end_visible())
     return;
 
-  m_control->core()->start(*m_downloadList.get_focus());
+  m_control->core()->download_list()->start(*m_view->focus());
+  m_view->set_last_changed();
 }
 
 void
 DownloadList::receive_stop_download() {
-  if (m_downloadList.get_focus() == m_downloadList.end())
+  if (m_view->focus() == m_view->end_visible())
     return;
 
-  if ((*m_downloadList.get_focus())->get_download().is_active())
-    m_control->core()->stop(*m_downloadList.get_focus());
+  if ((*m_view->focus())->variable()->get_value("state") == 1)
+    m_control->core()->download_list()->stop(*m_view->focus());
   else
-    m_downloadList.set_focus(m_control->core()->erase(m_downloadList.get_focus()));
+    m_control->core()->download_list()->erase(*m_view->focus());
+
+  m_view->set_last_changed();
+}
+
+void
+DownloadList::receive_close_download() {
+  if (m_view->focus() == m_view->end_visible())
+    return;
+
+  m_control->core()->download_list()->close(*m_view->focus());
+  m_view->set_last_changed();
 }
 
 void
 DownloadList::receive_view_download() {
-  if (m_downloadList.get_focus() == m_downloadList.end())
+  if (m_view->focus() == m_view->end_visible())
     return;
 
   if (m_uiDownload != NULL)
@@ -216,7 +236,7 @@ DownloadList::receive_view_download() {
 
   disable();
 
-  m_uiDownload = new Download(*m_downloadList.get_focus(), m_control);
+  m_uiDownload = new Download(*m_view->focus(), m_control);
 
   m_uiDownload->activate();
   m_uiDownload->get_bindings()[KEY_LEFT] = sigc::mem_fun(*this, &DownloadList::receive_exit_download);
@@ -231,6 +251,7 @@ DownloadList::receive_exit_download() {
   delete m_uiDownload;
   m_uiDownload = NULL;
 
+  m_view->set_last_changed();
   activate();
 
   m_control->display()->adjust_layout();
@@ -238,26 +259,27 @@ DownloadList::receive_exit_download() {
 
 void
 DownloadList::receive_next_priority() {
-  if (m_downloadList.get_focus() == m_downloadList.end())
+  if (m_view->focus() == m_view->end_visible())
     return;
 
-  (*m_downloadList.get_focus())->set_priority(((*m_downloadList.get_focus())->priority() + 1) % 4);
+  (*m_view->focus())->set_priority(((*m_view->focus())->priority() + 1) % 4);
 }
 
 void
 DownloadList::receive_prev_priority() {
-  if (m_downloadList.get_focus() == m_downloadList.end())
+  if (m_view->focus() == m_view->end_visible())
     return;
 
-  (*m_downloadList.get_focus())->set_priority(((*m_downloadList.get_focus())->priority() - 1) % 4);
+  (*m_view->focus())->set_priority(((*m_view->focus())->priority() - 1) % 4);
 }
 
 void
 DownloadList::receive_check_hash() {
-  if (m_downloadList.get_focus() == m_downloadList.end())
+  if (m_view->focus() == m_view->end_visible())
     return;
 
-  m_control->core()->check_hash(*m_downloadList.get_focus());
+  // Catch here?
+  m_control->core()->download_list()->check_hash(*m_view->focus());
 }
 
 void
@@ -274,7 +296,7 @@ DownloadList::receive_view_input(Input type) {
   m_windowTextInput->set_focus(true);
 
   if (type == INPUT_CHANGE_DIRECTORY) {
-    m_windowTextInput->get_input()->str() = m_control->variables()->get_string("directory");
+    m_windowTextInput->get_input()->str() = m_control->variable()->get_string("directory");
     m_windowTextInput->get_input()->set_pos(m_windowTextInput->get_input()->str().length());
   }
 
@@ -304,15 +326,15 @@ DownloadList::receive_exit_input(Input type) {
       break;
 
     case INPUT_CHANGE_DIRECTORY:
-      if (m_downloadList.get_focus() == m_downloadList.end())
+      if (m_view->focus() == m_view->end_visible())
 	throw torrent::input_error("No download in focus to change root directory.");
 
-      (*m_downloadList.get_focus())->variables()->set("directory", rak::trim(m_windowTextInput->get_input()->str()));
-      m_control->core()->push_log("New root dir \"" + (*m_downloadList.get_focus())->variables()->get_string("directory") + "\"");
+      (*m_view->focus())->variable()->set("directory", rak::trim(m_windowTextInput->get_input()->str()));
+      m_control->core()->push_log("New root dir \"" + (*m_view->focus())->variable()->get_string("directory") + "\" for torrent.");
       break;
 
     case INPUT_COMMAND:
-      m_control->variables()->process_command(m_windowTextInput->get_input()->str());
+      m_control->variable()->process_command(m_windowTextInput->get_input()->str());
       break;
     }
 
@@ -345,8 +367,7 @@ DownloadList::receive_change(Display d) {
 
 void
 DownloadList::receive_download_erased(core::Download* d) {
-  if (m_downloadList.get_focus() == m_downloadList.end() ||
-      *m_downloadList.get_focus() != d)
+  if (m_view->focus() == m_view->end_visible() || *m_view->focus() != d)
     return;
 
   if (m_uiDownload != NULL)
@@ -355,6 +376,26 @@ DownloadList::receive_download_erased(core::Download* d) {
   receive_next();
 }
 
+void
+DownloadList::receive_change_view(const std::string& name) {
+  core::ViewManager::iterator itr = m_control->view_manager()->find(name);
+
+  if (itr == m_control->view_manager()->end()) {
+    m_control->core()->push_log("Could not find view \"" + name + "\".");
+    return;
+  }
+
+  m_view = *itr;
+  m_view->sort();
+
+  ElementDownloadList* ui = dynamic_cast<ElementDownloadList*>(m_uiArray[DISPLAY_DOWNLOAD_LIST]);
+
+  if (ui == NULL)
+    throw torrent::client_error("DownloadList::receive_change_view(...) could not cast ui.");
+
+  ui->set_view(m_view);
+}
+
 void
 DownloadList::task_update() {
   m_windowLog->receive_update();
@@ -366,6 +407,7 @@ void
 DownloadList::setup_keys() {
   (*m_bindings)['\x13']        = sigc::mem_fun(*this, &DownloadList::receive_start_download);
   (*m_bindings)['\x04']        = sigc::mem_fun(*this, &DownloadList::receive_stop_download);
+  (*m_bindings)['\x0B']        = sigc::mem_fun(*this, &DownloadList::receive_close_download);
   (*m_bindings)['\x12']        = sigc::mem_fun(*this, &DownloadList::receive_check_hash);
   (*m_bindings)['+']           = sigc::mem_fun(*this, &DownloadList::receive_next_priority);
   (*m_bindings)['-']           = sigc::mem_fun(*this, &DownloadList::receive_prev_priority);
@@ -382,6 +424,14 @@ DownloadList::setup_keys() {
   (*m_bindings)[KEY_RIGHT]     = sigc::mem_fun(*this, &DownloadList::receive_view_download);
   (*m_bindings)['l']           = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_change), DISPLAY_LOG);
 
+  (*m_bindings)['1']           = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_change_view), "main");
+  (*m_bindings)['2']           = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_change_view), "name");
+  (*m_bindings)['3']           = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_change_view), "started");
+  (*m_bindings)['4']           = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_change_view), "stopped");
+  (*m_bindings)['5']           = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_change_view), "complete");
+  (*m_bindings)['6']           = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_change_view), "incomplete");
+  (*m_bindings)['7']           = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_change_view), "hashing");
+
   m_uiArray[DISPLAY_LOG]->get_bindings()[' '] = sigc::bind(sigc::mem_fun(*this, &DownloadList::receive_change), DISPLAY_DOWNLOAD_LIST);
 }
 
