@@ -37,6 +37,7 @@
 #ifndef RTORRENT_UI_ROOT_H
 #define RTORRENT_UI_ROOT_H
 
+#include <inttypes.h>
 #include "input/bindings.h"
 
 class Control;
@@ -63,6 +64,10 @@ public:
   void                set_down_throttle(unsigned int throttle);
   void                set_up_throttle(unsigned int throttle);
 
+  // Rename to raw or something, make base function.
+  void                set_down_throttle_i64(int64_t throttle) { set_down_throttle(throttle >> 10); }
+  void                set_up_throttle_i64(int64_t throttle)   { set_up_throttle(throttle >> 10); }
+
   void                adjust_down_throttle(int throttle);
   void                adjust_up_throttle(int throttle);
 
