@@ -43,7 +43,7 @@
 #include <rak/functional.h>
 #include <rak/path.h>
 #include <torrent/exceptions.h>
-#include <torrent/bencode.h>
+#include <torrent/object.h>
 
 #include "variable.h"
 #include "variable_map.h"
@@ -59,14 +59,14 @@ VariableMap::insert(const std::string& key, Variable* v) {
   iterator itr = base_type::find(key);
 
   if (itr != base_type::end())
-    throw torrent::internal_error("VariableMap::insert(...) tried to insert an already existing key.");
+    throw torrent::client_error("VariableMap::insert(...) tried to insert an already existing key.");
 
   base_type::insert(itr, value_type(key, v));
 }
 
 const VariableMap::mapped_type&
-VariableMap::get(const std::string& key) {
-  iterator itr = base_type::find(key);
+VariableMap::get(const std::string& key) const {
+  const_iterator itr = base_type::find(key);
 
   if (itr == base_type::end())
     throw torrent::input_error("Variable \"" + key + "\" does not exist.");
@@ -126,7 +126,7 @@ parse_unknown(std::string::const_iterator first, std::string::const_iterator las
 }
 
 std::string::const_iterator
-parse_args(std::string::const_iterator first, std::string::const_iterator last, VariableMap::mapped_type::List* dest) {
+parse_args(std::string::const_iterator first, std::string::const_iterator last, VariableMap::mapped_type::list_type* dest) {
   first = std::find_if(first, last, std::not1(variable_map_is_space()));
 
   while (first != last) {
@@ -198,15 +198,4 @@ VariableMap::process_file(const std::string& path) {
   return true;
 }
 
-void
-VariableMap::process_file_throw(const std::string& path) {
-  if (!process_file(path))
-    throw torrent::input_error("Could not open option file: " + path);
-}
-
-void
-VariableMap::process_file_nothrow(const std::string& path) {
-  process_file(path);
-}
-
 }
